# Token.pm (HCT::Lang::Token)
#
# Copyright 2008, 2009
# Slade Maurer
#
# This file is part of HCT, the Hardware Complexity Tool
#
# HCT is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# HCT is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU Lesser General Public License
# along with HCT.  If not, see <http://www.gnu.org/licenses/>.

=head1 NAME

HCT::Lang::Token - Definition of tokens.

=head1 DESCRIPTION

A token is a categorized block of text. The block of text corresponding to the token is known as a lexeme. 
A lexical analyzer divides a sequence of characters into tokens (tokenization) and categorizes them according to 
function, giving them meaning. A token can look like anything; it just needs to be a useful part of the structured 
text.

The object of token created by class C<HCT::Lang::Token> has such features: name, type and list of patterns. The lexemes 
will be gotten by lexer by using patterns.

The token name is an unique name that will be returned by lexer to parser.

The patterns are a list of patterns, that will be used for definition of the token name and token lexeme. All patterns 
described by regular expressions.

The token type is a number, that means position/preority, where the token have to be used. For instance, tokens can 
be divided on keywords, operators, numbers and identifiers, and the order in which lexer should use them 
will be: operators, numbers, keywords and identifiers. According to this the order can be: 
operators - 3, numbers - 2, keywrods - 1, identifiers - 0.

=cut

package HCT::Lang::Token;

our $VERSION = '0.7';

require Exporter;

use strict;
use warnings;

our @ISA    = qw(Exporter);
our @EXPORT = qw(
	EMPTY_TOKEN
);

use strict;
use warnings;

=head2 FUNCTIONS

The functions above will be exported by default.

=over 4

=item B<EMPTY_TOKEN>

When the lexer reach the end of input, it returns the C<''> empty 
token with an C<undef> value.

=cut

sub EMPTY_TOKEN
{
	return ('', undef);
}

=back

=cut

=head2 CONSTRUCTOR

=over 4

=item B<new ()>

Creates a new C<HCT::Language::Token> object.

=back

=cut

sub new
{
	my $inv = shift;
	my $class = ref($inv) || $inv;
	my $token = {
		name => undef,
		patterns => [ ],
		type => undef,
		@_
	};

	# Validation...
	HCT::ERROR ("Name of token is undefined.\n") and return undef
		if (!defined $token->{'name'});
	HCT::ERROR ("Bad definition of token '$token->{'name'}'.\n") and return undef
		if (!defined $token->{'patterns'} || ref($token->{'patterns'}) ne 'ARRAY');
	HCT::ERROR ("Token '$token->{name} has no lexemes'.\n") and return undef
		if (scalar(@{$token->{'patterns'}}) == 0);

	# Create new token object
	bless $token, $class;

	# Process all patterns. Some of them are not a regular expressions.
	for (my $i=0; $i<scalar($token->get_patterns); $i++) {
		if (ref ($token->{'patterns'}->[$i]) ne 'Regexp') {
			$token->{'patterns'}->[$i] = quotemeta ($token->{'patterns'}->[$i]);
			$token->{'patterns'}->[$i] = qr/$token->{'patterns'}->[$i]/;
		}
	}

	return $token; 
}

=head2 METHODS

=over 4

=item B<get_patterns ()>

Returns a list of token patterns.

=cut

sub get_patterns
{
	return @{$_[0]->{'patterns'}};
}

=item B<get_name ()>

Returns token name.

=cut

sub get_name {
	return $_[0]->{'name'};
}

=item B<get_type ()>

Returns token type.

=cut

sub get_type
{
	return $_[0]->{'type'};
}

=back
=cut

1;

