# lexer.pm (HCT::lang::hdl::cdl::lexer) - CDL Lexer.
#
# Copyright 2008, 2009
# Slade Maurer
#
# This file is part of HCT, the Hardware Complexity Tool
#
# HCT is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# HCT is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU Lesser General Public License
# along with HCT.  If not, see <http://www.gnu.org/licenses/>.

package HCT::lang::hdl::cdl::lexer;

use base 'HCT::Lang::LexerDriver';

use HCT::Constants;
use HCT::Lang::Token;

use strict;
use warnings;

use constant {
	operator => 4,
	number => 3,
	keyword => 2,
	identifier => 1,
	misc => 0,
};

sub reserve_tokens
{
	my $lexer = shift;
	
	$lexer->reserve (operator,
		['==', [ '==' ]],
		['?', ['?']],
		[':', [':']],
		['+', ['+']],
		['*', ['*']],
		['/', ['/']],
		['%', ['%']],
		['&', [qr/\&(?!\&)/]],
		['|', [qr/\|(?!\|)/]],
		['^', ['^']],
		['<', [qr/\<(?!\<)/]],
		['>', [qr/\>(?!\>)/]],
		['||', ['||']],
		['&&', ['&&']],
		['!=', ['!=']],
		['<=', ['<=']],
		['>=', ['>=']],
		['<<', ['<<']],
		['>>', ['>>']],
		['!', ['!']],
		['~', ['~']],
		['-', ['-']],
	);

	$lexer->reserve (number,
		['TOKEN_SIZED_INT', [qr/[0-9]+(?![x|0-9])/,qr/0x[0-9a-fA-FxXzZ?][0-9a-fA-FxXzZ?_]*/]],
	);

	$lexer->reserve (keyword,
		['TOKEN_STRING', [qr/\"[^"]*\"/]],
		['TOKEN_TEXT_CONSTANT', ['constant']],
		['TOKEN_TEXT_STRUCT', ['struct']],
		['TOKEN_TEXT_FSM', ['fsm']],
		['TOKEN_TEXT_ONE_HOT', ['one_hot']],
		['TOKEN_TEXT_ONE_COLD', ['one_cold']],
		['TOKEN_TEXT_SCHEMATIC', ['schematic']],
		['TOKEN_TEXT_SYMBOL', [ 'symbol']],
		['TOKEN_TEXT_PORT', [ 'port']],
		['TOKEN_TEXT_LINE', [ 'line']],
		['TOKEN_TEXT_FILL', [ 'fill']],
		['TOKEN_TEXT_OVAL', [ 'oval']],
		['TOKEN_TEXT_OPTION', [ 'option']],
		['TOKEN_TEXT_PRECLOCK', [ 'preclock']],
		['TOKEN_TEXT_REGISTER', [ 'register']],
		['TOKEN_TEXT_ASSERT', [ 'assert']],
		['TOKEN_TEXT_COVER', [ 'cover']],
		['TOKEN_TEXT_INCLUDE', [ 'include']],
		['TOKEN_TEXT_TYPEDEF', [ 'typedef']],
		['TOKEN_TEXT_STRING', [ 'string']],
		['TOKEN_TEXT_BIT', [ 'bit']],
		['TOKEN_TEXT_INTEGER', [ 'integer']],
		['TOKEN_TEXT_ENUM', [ 'enum']],
		['TOKEN_TEXT_EXTERN', [ 'extern']],
		['TOKEN_TEXT_MODULE', [ 'module']],
		['TOKEN_TEXT_INPUT', [ 'input']],
		['TOKEN_TEXT_OUTPUT', [ 'output']],
		['TOKEN_TEXT_PARAMETER', [ 'parameter']],
		['TOKEN_TEXT_TIMING', [ 'timing']],
		['TOKEN_TEXT_TO', [ 'to']],
		['TOKEN_TEXT_FROM', [ 'from']],
		['TOKEN_TEXT_BUNDLE', [ 'bundle']],
		['TOKEN_TEXT_DEFAULT', [ 'default']],
		['TOKEN_TEXT_CLOCK', [ 'clock']],
		['TOKEN_TEXT_RISING', [ 'rising']],
		['TOKEN_TEXT_FALLING', [ 'falling']],
		['TOKEN_TEXT_GATED_CLOCK', [ 'gated_clock']],
		['TOKEN_TEXT_RESET', [ 'reset']],
		['TOKEN_TEXT_ACTIVE_LOW', [ 'active_low']],
		['TOKEN_TEXT_ACTIVE_HIGH', [ 'active_high']],
		['TOKEN_TEXT_CLOCKED', [ 'clocked']],
		['TOKEN_TEXT_COMB', [ 'comb']],
		['TOKEN_TEXT_NET', [ 'net']],
		['TOKEN_TEXT_FOR', [ 'for']],
		['TOKEN_TEXT_IF', [ 'if']],
		['TOKEN_TEXT_ELSIF', [ 'elsif']],
		['TOKEN_TEXT_ELSE', [ 'else']],
		['TOKEN_TEXT_FULLSWITCH', [ 'full_switch']],
		['TOKEN_TEXT_PARTSWITCH', [ 'part_switch']],
		['TOKEN_TEXT_PRIORITY', [ 'priority']],
		['TOKEN_TEXT_CASE', [ 'case']],
		['TOKEN_TEXT_BREAK', [ 'break']],
		['TOKEN_TEXT_SIZEOF', [ 'sizeof']],
		['TOKEN_TEXT_PRINT', [ 'print']],
	);
	
	$lexer->reserve (identifier,
		['TOKEN_USER_ID', [qr/[a-zA-Z_][a-zA-Z0-9_]*/]]
	);

	$lexer->reserve (misc,
		[';',[';']],
		['(',['(']],
		[')',[')']],
		['[',['[']],
		[']',[']']],
		['=',['=']],
		['@',['@']],
		['#',['#']],
		[',',[',']],
		['.',['.']],
		['{',['{']],
		['}',['}']],
	);	
}

sub event_eol
{
	my $lexer = shift;

	if (defined $lexer->callback->module->{current}) {
		$lexer->callback->module->{current}->{metrics}->{SLOC} += 1;
	} else {
		$lexer->{handler}->{lang}->{metrics}->[0]->[1]->{'SLOC'} += 1; 
	}
}

sub stop
{
	my $lexer = shift;

	if ($lexer->input->eof && $lexer->input->close) {
		return $lexer->push_token ('TOKEN_EOF', undef);
	}

	return $lexer->push_token (EMPTY_TOKEN);
}

sub skip_comment
{
	my $lexer = shift;
	my $multiline_comment = FALSE;

	# linecomment
	if (defined $lexer->match (qr/\s*\/\//) ) {
		
		if (defined $lexer->callback->module->{current}) {
			$lexer->callback->module->{current}->{metrics}->{COMMENT_LINES} += 1;
		} else {
			$lexer->{handler}->{lang}->{metrics}->[0]->[1]->{COMMENT_LINES} += 1; 
		}
		
		
		$lexer->linepos ($lexer->linelen);
		return TRUE
	}

	# multiline comment
	if (defined $lexer->match(qr/\s*\/\*/) ) {
		$multiline_comment = TRUE;
	}
	
	while ($multiline_comment == TRUE) {

		if (defined $lexer->callback->module->{current}) {
			$lexer->callback->module->{current}->{metrics}->{COMMENT_LINES} += 1;
		} else {
			$lexer->{handler}->{lang}->{metrics}->[0]->[1]->{COMMENT_LINES} += 1; 
		}

		if (defined $lexer->match (qr/(.*?)\*\//) ) {
			return TRUE;
		}
		$lexer->linepos ($lexer->linelen);
		$lexer->move;
	}

	return FALSE;
}

1;



