package HCT::lang::hdl::vhdl::lexer;

use base 'HCT::Lang::LexerDriver';

use HCT::Constants;
use HCT::Lang::Token;

use strict;
use warnings;

use constant {
	list		=> 4,
	operator 	=> 3,
	number		=> 2,
	keyword		=> 1,
	id		=> 0,
};

sub reserve_tokens
{
	my $lexer = shift;

	$lexer->reserve (
		operator,

		[ 'Box'		, [ qr{<>} ] ],
		[ 'VarAsgn'	, [ qr{:=} ] ],
		[ 'LESym'	, [ qr{<=} ] ],
		[ 'DoubleStar'	, [ qr{\*\*} ] ],
		[ 'Arrow'	, [ qr{=>} ] ],
		[ 'GESym'	, [ qr{>=} ] ],
		[ 'NESym'	, [ qr{/=} ] ],

		[ 'Ampersand'	, [ qr{&} ] ],
		[ 'Apostrophe'	, [ qr{'} ] ],
		[ 'LeftParen'	, [ q{(} ] ],
		[ 'RightParen'	, [ qr{\)} ] ],
		[ 'Star'	, [ qr{\*} ] ],
		[ 'Plus'	, [ qr{\+} ] ],
		[ 'Comma'	, [ qr{,} ] ],
		[ 'Minus'	, [ qr{-} ] ],
		[ 'Colon'	, [ qr{:} ] ],
		[ 'Semicolon'	, [ qr{\;} ] ],
		[ 'LTSym'	, [ qr{<} ] ],
		[ 'EQSym'	, [ qr{=} ] ],
		[ 'GTSym'	, [ qr{>} ] ],
		[ 'Bar'		, [ qr{\|} , qr{!} ] ],
		[ 'Dot'		, [ qr{\.} ] ],
		[ 'Slash'	, [ qr{/} ] ],
		
	);

	$lexer->reserve (
		id,
		['Identifier'	, [ qr/[a-zA-Z][a-zA-Z0-9_]*/ ] ],
	);

	$lexer->reserve (
		number,
		['BasedInt'	, [ qr{[0-9]+\#[0-9a-fA-F]+\#}] ],
		['BasedReal'	, [ qr{[0-9]+\#[0-9a-fA-F_]+\#} ] ],
		['DecimalInt'	, [ qr{[0-9]+} , qr{[0-9]+(e|E)(\+[0-9]+|[0-9]+)} ] ],
		['DecimalReal'	, [ qr{[0-9]+\.[0-9]+} , qr{[0-9]+\.[0-9]+(e|E)((\+|\-)[0-9]+|[0-9]+)} ] ],
	);

	$lexer->reserve (
		list,
		['CharacterLit'	, [ qr{'[\w\W\S]'} ] ],
		['StringLit'	, [ qr{"[\w\W\S]*"} ] ],
		['BitStringLit'	, [ qr{(b|B)"[0-1_]+"} , qr{(x|X)"[0-9a-fA-F]+"} ] , qr{(o|O)\[0-7]+"} ],
	);

	$lexer->reserve (
		keyword,

		# - A
		['ABS'		, [ qr/abs(?!\w)/	 ] ],
		['ACCESS'	, [ qr/access(?!\w)/	 ] ],
		['AFTER'	, [ qr/after(?!\w)/	 ] ],
		['ALIAS'	, [ qr/alias(?!\w)/	 ] ],
		['ALL'		, [ qr/all(?!\w)/	 ] ],
		['AND'		, [ qr/and(?!\w)/ ] ],
		['ARCHITECTURE'	, [ qr/architecture(?!\w)/ ] ],
		['ARRAY'	, [ qr/array(?!\w)/ ] ],
		['ASSERT'	, [ qr/assert(?!\w)/ ] ],
		['ATTRIBUTE'	, [ qr/attribute(?!\w)/ ] ],

		# - B
		['BEGIN_'	, [ qr/begin(?!\w)/ ] ],
		['BLOCK'	, [ qr/block(?!\w)/ ] ],
		['BODY'		, [ qr/body(?!\w)/ ] ],
		['BUFFER'	, [ qr/buffer(?!\w)/ ] ],
		['BUS'		, [ qr/bus(?!\w)/ ] ],

		# - C
		['CASE'		, [ qr/case(?!\w)/ ] ],
		['COMPONENT'	, [ qr/component(?!\w)/ ] ],
		['CONFIGURATION', [ qr/configuration(?!\w)/ ] ],
		['CONSTANT'	, [ qr/constant(?!\w)/ ] ],

		# - D
		['DISCONNECT'	, [ qr/disconnect(?!\w)/ ] ],
		['DOWNTO'	, [ qr/downto(?!\w)/ ] ],

		# - E
		['ELSE'		, [ qr/else(?!\w)/ ] ],
		['ELSIF'	, [ qr/elsif(?!\w)/ ] ],
		['END_'		, [ qr/end(?!\w)/ ] ],
		['ENTITY'	, [ qr/entity(?!\w)/ ] ],
		['EXIT'		, [ qr/exit(?!\w)/ ] ],

		# - F
		['FILE_'	, [ qr/file(?!\w)/ ] ],
		['FOR'		, [ qr/for(?!\w)/ ] ],
		['FUNCTION'	, [ qr/function(?!\w)/ ] ],

		# - G
		['GENERATE'	, [ qr/generate(?!\w)/ ] ],
		['GENERIC'	, [ qr/generic(?!\w)/ ] ],
		['GUARDED'	, [ qr/guarded(?!\w)/ ] ],

		# - I
		['IF'		, [ qr/if(?!\w)/ ] ],
		['IN'		, [ qr/in(?!\w)/ ] ],
		['INOUT'	, [ qr/inout(?!\w)/ ] ],
		['IS'		, [ qr/is(?!\w)/ ] ],

		# - L
		['LABEL'	, [ qr/label(?!\w)/ ] ],
		['LIBRARY'	, [ qr/library(?!\w)/ ] ],
		['LINKAGE'	, [ qr/linkage(?!\w)/ ] ],
		['LOOP'		, [ qr/loop(?!\w)/ ] ],

		# - M
		['MAP'		, [ qr/map(?!\w)/ ] ],
		['MOD'		, [ qr/mod(?!\w)/ ] ],

		# - N
		['NAND'		, [ qr/nand(?!\w)/ ] ],
		['NEW'		, [ qr/new(?!\w)/ ] ],
		['NEXT'		, [ qr/next(?!\w)/ ] ],
		['NOR'		, [ qr/nor(?!\w)/ ] ],
		['NOT'		, [ qr/not(?!\w)/ ] ],
		['NULL_'	, [ qr/null(?!\w)/ ] ],

		# - O
		['OF'		, [ qr/of(?!\w)/ ] ],
		['ON'		, [ qr/on(?!\w)/ ] ],
		['OPEN'		, [ qr/open(?!\w)/ ] ],
		['OR'		, [ qr/or(?!\w)/ ] ],
		['OTHERS'	, [ qr/others(?!\w)/ ] ],
		['OUT'		, [ qr/out(?!\w)/ ] ],

		# - P
		['PACKAGE'	, [ qr/package(?!\w)/ ] ],
		['PORT'		, [ qr/port(?!\w)/ ] ],
		['PROCEDURE'	, [ qr/procedure(?!\w)/ ] ],
		['PROCESS'	, [ qr/process(?!\w)/ ] ],

		# - R
		['RANGE'	, [ qr/range(?!\w)/ ] ],
		['RECORD'	, [ qr/record(?!\w)/ ] ],
		['REGISTER'	, [ qr/register(?!\w)/ ] ],
		['REM'		, [ qr/rem(?!\w)/ ] ],
		['REPORT'	, [ qr/report(?!\w)/ ] ],
		['RETURN'	, [ qr/retur(?!\w)/ ] ],

		# - S
		['SELECT'	, [ qr/select(?!\w)/ ] ],
		['SEVERITY'	, [ qr/severity(?!\w)/ ] ],
		['SIGNAL'	, [ qr/signal(?!\w)/ ] ],
		['SUBTYPE'	, [ qr/subtype(?!\w)/ ] ],

		# - T
		['THEN'		, [ qr/then/ ] ],
		['TO'		, [ qr/to(?!\w)/ ] ],
		['TRANSPORT'	, [ qr/transport(?!\w)/ ] ],
		['TYPE'		, [ qr/type(?!\w)/ ] ],

		# - U
		['UNITS'	, [ qr/units(?!\w)/ ] ],
		['UNTIL'	, [ qr/until(?!\w)/ ] ],
		['USE'		, [ qr/use(?!\w)/ ] ],

		# - V
		['VARIABLE'	, [ qr/variable(?!\w)/ ] ],

		# - W
		['WAIT'		, [ qr/wait(?!\w)/ ] ],
		['WHEN'		, [ qr/when(?!\w)/ ] ],
		['WHILE'	, [ qr/while(?!\w)/ ] ],
		['WITH'		, [ qr/with(?!\w)/ ] ],

		# - X
		['XOR'		, [ qr/xor(?!\w)/ ] ],
	);

}

sub event_eol
{
	my $lexer = shift;
	
	if (defined $lexer->callback->module->{current}) {
		$lexer->callback->module->{current}->{metrics}->{SLOC} += 1;
	} else {
		$lexer->{handler}->{lang}->{metrics}->[0]->[1]->{'SLOC'} += 1; 
	}
}

sub skip_comment
{
	my $lexer = shift;
	my $multiline_comment = FALSE;

	# linecomment
	if (defined $lexer->match (qr/\s*\-\-/) ) {
		
		if (defined $lexer->callback->module->{current}) {
			$lexer->callback->module->{current}->{metrics}->{COMMENT_LINES} += 1;
		} else {
			$lexer->{handler}->{lang}->{metrics}->[0]->[1]->{COMMENT_LINES} += 1; 
		}
		
		
		$lexer->linepos ($lexer->linelen);
		return TRUE
	}

	return FALSE;
}

sub stop
{
	my $lexer = shift;

	return $lexer->push_token (EMPTY_TOKEN);
}



1;
